import pandas as pd


def codonsToAminos(df):
    df_a = pd.DataFrame(
        columns=['Position', 'K', 'N', 'T', 'R', 'S', 'I', 'M', 'Q', 'H', 'P', 'L', 'E', 'D', 'A', 'G', 'V', '*', 'Y',
                 'C', 'W', 'F', 'Passage', 'Strain', 'Repetition'])
    df_a['Position'] = df.Position
    df_a['Passage'] = df.Passage
    df_a['Strain'] = df.Strain
    df_a['Repetition'] = df.Repetition
    df_a['K'] = df.AAA + df.AAG
    df_a['N'] = df.AAC + df.AAT
    df_a['T'] = df.ACA + df.ACC + df.ACG + df.ACT
    df_a['R'] = df.AGA + df.AGG + df.CGA + df.CGC + df.CGG + df.CGT
    df_a['S'] = df.AGC + df.AGT + df.TCA + df.TCC + df.TCG + df.TCT
    df_a['I'] = df.ATA + df.ATC + df.ATT
    df_a['M'] = df.ATG
    df_a['Q'] = df.CAA + df.CAG
    df_a['H'] = df.CAC + df.CAT
    df_a['P'] = df.CCA + df.CCC + df.CCG + df.CCT
    df_a['L'] = df.CTA + df.CTC + df.CTG + df.CTT + df.TTA + df.TTG
    df_a['E'] = df.GAA + df.GAG
    df_a['D'] = df.GAC + df.GAT
    df_a['A'] = df.GCA + df.GCC + df.GCG + df.GCT
    df_a['G'] = df.GGA + df.GGC + df.GGG + df.GGT
    df_a['V'] = df.GTA + df.GTC + df.GTG + df.GTT
    df_a['*'] = df.TAA + df.TAG + df.TGA
    df_a['Y'] = df.TAC + df.TAT
    df_a['C'] = df.TGC + df.TGT
    df_a['W'] = df.TGG
    df_a['F'] = df.TTC + df.TTT
    return df_a



import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    import numpy as np
    import pandas as pd
    import math
    import itertools
    import matplotlib.pyplot as plt
    from matplotlib import gridspec
    from tslearn.clustering import TimeSeriesKMeans
    from pyts.decomposition import SingularSpectrumAnalysis
    from tslearn.utils import to_time_series
    from sklearn.manifold import TSNE
    from src.Format_DF import totalExperimentsToPassages, formatStrains, experimentToPositions, totalExperimentsToPositions


def clustering(data,num_of_clus,metrica):
    """Function that returns the clustering
     Parameters:
     data: Array of data where each row is the entropy path at a position
     num_of_clus: Number of clusters
     metric: Metric to be used by TimeSeriesKMeans

     Returns array of size (quantity_of_ts,) and tslearn.clustering object
    """


    tslearn_time_series = to_time_series(data)

    km = TimeSeriesKMeans(n_clusters=num_of_clus, metric=metrica,random_state=0)
    labels_clus= km.fit_predict(tslearn_time_series)

    barycenters = km.cluster_centers_

    return labels_clus, barycenters

def listPositions(labels_clus, num_of_clus):
    """Function that returns a list with both objects and num_of_clus
     Parameters:
     labels_clus: The labels returned by the clustering method
     num_of_clas: Number of clusters

     Returns: List with as many objects as num_of_clus where each component is a list with the positions of each cluster
    """

    pos=labels_clus.size
    positions=[]

    for j in range(num_of_clus):
        aux=[]
        for i in range(pos):

            if labels_clus[i]==j: aux.append(i)

        positions.append(aux)

    return positions




def plotClustersCurves(data,pos, bary,nroCluster,directory):
    """Function that plots the trajectories of the entropies of arr, indicated in pos
     Parameters:
     data: Array of data where each row is the entropy path at a position
     pos: Positions to plot (labels returned by the tslearn.clustering object)
     bary: Center of gravity of the clusters
    """

    x=np.arange(data[0].size)

    total_fig = bary.shape[0]

    if total_fig >= 3:
        cols = 3
        rows = int(math.ceil(total_fig / cols))
    else:
        cols = 1
        rows = 2


    gs = gridspec.GridSpec(rows, cols)
    fig = plt.figure()

    for cent in range(bary.shape[0]):
        ax = fig.add_subplot(gs[cent])
        for p in pos[cent]:
                y=data[p,:]
                ax.plot(x, y, 'k', alpha=0.1)
        ax.plot(x, bary[cent], 'r', linewidth=2)

    fig.set_size_inches(10, 8)
    fig.tight_layout()
    plt.savefig(directory+'/Curves'+str(nroCluster))
    plt.close()


def plotClustersDimRed(data_DR,pos, bary,nroCluster,directory):
    """Function that plots in the 2D plane the positions according to the cluster to which they belong (indicated in pos)
     Parameters:
     data_DR: Array of data with the transformation components (PCA, tSNE, etc)
     pos: List with as many objects as the number of clusters with all positions belonging to the cluster (corresponds to positions returned by listPositions)
     bary: barycenters of the clusters (corresponds to barycenters returned by clustering. It is an array with a time series for each barycenter)
    """

    fig, ax = plt.subplots()
    i=0

    for cent in range(bary.shape[0]):

        x=data_DR[pos[cent],0]
        y=data_DR[pos[cent],1]
        i+=1
        ax.scatter(x, y, alpha=0.3)

    fig.set_size_inches(12, 10)
    plt.show()


def plotClustersCentDimRed(data_DR,pos, bary_DR,nroCluster,directory):
    """Function that plots in the plane of reduced dimension the positions according to the cluster to which they belong (indicated in pos)
     Parameters:
     data_DR: Array of data with the transformation components (PCA, tSNE, etc)
     pos: List with as many objects as the number of clusters with all positions belonging to the cluster (corresponds to positions returned by listPositions)
     bary: Center of gravity of the clusters
    """

    fig, ax = plt.subplots()
    i=0

    for cent in range(bary_DR.shape[0]):
        x=data_DR[pos[cent],0]
        y=data_DR[pos[cent],1]
        i+=1
        ax.scatter(x, y, alpha=0.3)
        ax.scatter(bary_DR[[cent],0],bary_DR[[cent],1], marker='x', c='k', s=60)


    fig.set_size_inches(12, 10)
    plt.show()



def getClustering(list_df_entropy, metrica, perp=3, lr=100):

    number_clusters=[2,4,6] #number of clusters to plot

    df_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "2", "4", "6"]) #create data frame to save information

    array_data_Pos, array_Pos_Info, array_Pos_Strain=totalExperimentsToPositions(list_df_entropy)

    array_Data_Pos_TSNE = TSNE(perplexity=perp, learning_rate=lr, random_state=111).fit_transform(array_data_Pos) #t-SNE with every position

    df_Aux["tsne_1"]=array_Data_Pos_TSNE[:,0] #save in dataframe
    df_Aux["tsne_2"]=array_Data_Pos_TSNE[:,1]

    df_I = pd.DataFrame([sub.split(",") for sub in array_Pos_Info]) #split info column
    df_I.columns=["Strain", "Repetition", "Position"]  #create tree columns with this info

    pos_s = []
    bary_s = []

    for c in number_clusters:
        labels_clust, bary = clustering(array_data_Pos,c,metrica) #calls clustering
        list_posiciones = listPositions(labels_clust, c)
        pos_s.append(list_posiciones)
        bary_s.append(bary)
        df_Aux[str(c)] = labels_clust #add label in dataframe

    df_T = pd.concat([df_Aux, df_I], axis=1)



    return array_data_Pos, df_T, pos_s, bary_s

def getClusteringSSA(list_df_entropy, metrica, perp=3, lr=100, ssa_components=2):
    #same as function above usingin in this case SingularSpectrumAnalysis to transform array_data_Pos
    number_clusters=[2,4,6]

    df_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "2", "4", "6"])

    #Format all data to obtain an array where each row is a time series for each position
    array_data_Pos, array_Pos_Info, array_Pos_Strain=totalExperimentsToPositions(list_df_entropy)

    #SSA descomposition to all time series
    transformer = SingularSpectrumAnalysis(window_size=ssa_components)
    array_data_Pos_SSA = transformer.transform(array_data_Pos)
    array_data_Pos_SSA_firstComp = array_data_Pos_SSA[:,0,:]


    array_data_Pos_SSA_TSNE = TSNE(perplexity=perp, learning_rate=lr, random_state=111).fit_transform(array_data_Pos_SSA_firstComp)

    df_Aux["tsne_1"]=array_data_Pos_SSA_TSNE[:,0]
    df_Aux["tsne_2"]=array_data_Pos_SSA_TSNE[:,1]

    df_I = pd.DataFrame([sub.split(",") for sub in array_Pos_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    pos_s = []
    bary_s = []

    for c in number_clusters:
        labels_clust, bary = clustering(array_data_Pos_SSA_firstComp,c,metrica)
        list_posiciones = listPositions(labels_clust, c)
        pos_s.append(list_posiciones)
        bary_s.append(bary)
        df_Aux[str(c)] = labels_clust

    df_T = pd.concat([df_Aux, df_I], axis=1)

    return array_data_Pos, df_T, pos_s, bary_s

def plot_confusion_matrix(cm,nroCluster,directory, classes,
                          normalize=False,
                          title='Confusion matrix',
                          cmap=plt.cm.Blues):
    """
    This function prints and plots the confusion matrix.
    Normalization can be applied by setting `normalize=True`.
    """

    plt.imshow(cm, interpolation='nearest', cmap=cmap)
    plt.title(title)
    plt.colorbar()
    tick_marks = np.arange(len(classes))
    tick_marks_x = np.arange(len(classes)-2)
    plt.xticks(tick_marks_x, classes[:-2], rotation=45)
    plt.yticks(tick_marks, classes)


    thresh = cm.max() / 2.
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        plt.text(j, i, cm[i, j],
                 horizontalalignment="center",
                 color="white" if cm[i, j] > thresh else "black")

    plt.tight_layout()
    plt.ylabel('New cluster label')
    plt.xlabel('Old cluster label')
    plt.savefig(directory + '/confMatrix' + str(nroCluster) + '.png')
    plt.close()




import numpy as np
import pandas as pd


def experimentToPassages(df):
    # Function that takes the experiment (dataframe df) and converts each passage into a row

    passages = df['Passage'].unique()  # get passages

    info_df = str(df['info'].iloc[0])  # get info
    strain_df = info_df.split(',')[0]
    repetition_df = info_df.split(',')[1]

    arrayDF_P = []
    arrayInfo = []
    arrayStrain = []

    for p in passages:  # every passage

        dfP = df[df.Passage.eq(p)]  # get dataframe for this passage
        dfPE = dfP[['Entropy']]  # get entropy
        passageArray = np.array(dfPE)  # convert to numpy array
        pA = np.squeeze(np.array([passageArray]))  # reduce dimension
        arrayDF_P.append(pA)  # add pasage to array
        arrayInfo.append(str(strain_df) + ',' + str(repetition_df) + ',' + str(p))
        arrayStrain.append(str(strain_df))
    return np.array(arrayDF_P), np.array(arrayInfo), np.array(arrayStrain)


def totalExperimentsToPassages(parameters):
    # Function that takes all the experiments you have loaded in parameters and converts each experiment to one row per passage

    # Arrays where the passages are stored as rows and the info
    arrayT_DF_P = []
    arrayT_Info = []
    arrayT_Strain = []

    for df in parameters:  # every experiment
        arrayP, arrayI, arrayS = experimentToPassages(df)  # convert each passage into a row
        arrayT_DF_P.append(arrayP)
        arrayT_Info.append(arrayI)
        arrayT_Strain.append(arrayS)

    totalArrayData = np.concatenate(arrayT_DF_P)
    totalArrayInfo = np.concatenate(arrayT_Info)
    totalArrayStrain = np.concatenate(arrayT_Strain)

    return totalArrayData, totalArrayInfo, totalArrayStrain


def experimentToPositions(df):
    # Function that takes the experiment (dataframe df) and converts each position into a row
    # same as experimentsToPassage but with positions

    positions = df['Position'].unique()  # get positions

    info_df = str(df['info'].iloc[0])  # get info
    strain_df = info_df.split(',')[0]
    repetition_df = info_df.split(',')[1]

    arrayDF_P = []
    arrayInfo = []
    arrayStrain = []

    for p in positions:
        dfP = df[df.Position.eq(p)]
        dfPE = dfP[['Entropy']]
        positionArray = np.array(dfPE)

        pA = np.squeeze(np.array([positionArray]))
        arrayDF_P.append(pA)
        arrayInfo.append(str(strain_df) + ',' + str(repetition_df) + ',' + str(p))
        arrayStrain.append(str(strain_df))
    return np.array(arrayDF_P), np.array(arrayInfo), np.array(arrayStrain)


def totalExperimentsToPositions(parameters):
    # same as totalExperimentsToPassages but with positions
    arrayT_DF_P = []
    arrayT_Info = []
    arrayT_Strain = []

    for df in parameters:
        arrayP, arrayI, arrayS = experimentToPositions(df)
        arrayT_DF_P.append(arrayP)
        arrayT_Info.append(arrayI)
        arrayT_Strain.append(arrayS)

    totalArrayData = np.concatenate(arrayT_DF_P)
    totalArrayInfo = np.concatenate(arrayT_Info)
    totalArrayStrain = np.concatenate(arrayT_Strain)

    return totalArrayData, totalArrayInfo, totalArrayStrain


def formatStrains(strainT):
    # Function that takes the array of strains and assigns each one an int from 0
    # It also calculates the value that the perplexity parameter of tSNE should take according to the number of points of each stra

    # Pass the strains of the loaded experiments to dataframe
    df = pd.DataFrame(strainT, columns=["Strain"])
    df = df.assign(StrainNumber=df['Strain'])

    strainsU = df.Strain.unique()  # getthe number of passages of each strain
    dataRows = []
    i = 0
    for s in strainsU:
        n = df[df.Strain.eq(s)].count()
        dataRows.append(n)
        df.loc[df['Strain'] == s, 'StrainNumber'] = i
        i += 1
    arNum = np.array(dataRows)

    return arNum, df


import matplotlib.pyplot as plt
import os


def plotEntropyPosition(position, dataFrame,strain,repetition,color = 'red'):
    pathPip = str(os.path.dirname(plt.__file__)).split('matplotlib')[0]
    passage=dataFrame.loc[dataFrame.Position==position,'Passage'] #get passage information for that position
    entropy=dataFrame.loc[dataFrame.Position==position,'Entropy'] #get entropy information for that position
    fig1, ax = plt.subplots()
    ax.plot(passage, entropy,color=color) #plot image
    #labels
    ax.set_xlabel('Passages')
    ax.set_ylabel('Entropy')
    ax.set_title("Entropy over passages for position "+str(position))
    plt.grid()
    name =pathPip+ 'src/plot_images/Entropy_' + str(position) +'_'+strain+'_'+repetition+ '.png'
    plt.savefig(name)
    plt.close()


def plotEntropiesPosition(position, dataFrames): #plot entropy * position for every experiment in PCA
    pathPip = str(os.path.dirname(plt.__file__)).split('matplotlib')[0]
    ffig1, ax = plt.subplots()
    for df in dataFrames: #every experiment
        strain = df.Strain.unique()[0]
        repetition = df.Repetition.unique()[0]
        passage=df.loc[df.Position==position,'Passage'] #get info
        entropy = df.loc[df.Position == position, 'Entropy'] #get info
        line,=ax.plot(passage, entropy)
        auxLabel = strain + '-' +str(repetition) #labels
        line.set_label(auxLabel)
        ax.legend()
    #labels
    ax.set_xlabel('Passages')
    ax.set_ylabel('Entropy')
    ax.set_title("Entropy over passages for position " + str(position))
    plt.grid()
    name = pathPip+'src/plot_images/Entropy_' + str(position) +'.png'
    plt.savefig(name)
    plt.close()

    
    
    




import pandas as pd
import numpy as np
import os
import glob
from src.NumericalSort import numericalSort
import re

def readData(parameters):
    i = 0
    strainControl = ''
    repetitionControl = ''
    path=parameters[0]
    fvp=parameters[1]
    lvp=parameters[2]

    df_list = []
    for filename in sorted(glob.glob(os.path.join(path, '*.csv')) , key=numericalSort): #every csv in directory sorted
        #checks name of .csv
        auxInfo = filename.split('/')
        auxInfo = auxInfo[len(auxInfo)-1]
        auxInfo = auxInfo.split('_')
        auxPassage = auxInfo[1]
        strain = auxInfo[2]
        auxRepetition = auxInfo[3]
        repetition = re.sub("\D+", '', auxRepetition)
        passage = re.sub("\D+",'',auxPassage)
        if i == 0: # check only one type of strain and repetition
            strainControl = strain
            repetitionControl = repetition
            i = i +1
        if repetition != repetitionControl or strainControl != strain:
            raise Exception("Error, there are csv files from other experiment")
        df=pd.read_csv(filename) #get csv
        df['Passage']=passage #passage information is csv passage
        df_list.append(df) #add dataframe to array



    df_T = pd.concat(df_list) #concat all DataFrame passages in one
    df_T['Strain']=strain #add info of strain
    df_T['Repetition']=repetition #add info of repetition
    df_T['info'] = df_T.apply(lambda row: str(row.Strain) + ',' + str(row.Repetition) + "," + str(row.Passage) + "," + str(row.Position),axis=1) #create column info


    positions = df_T['Position'].unique() #get positions
    lvpAux = positions[len(positions) - 1] #get last positions of experiment
    if lvp ==0 or lvpAux < lvp: #control lvp value
        lvp = lvpAux
    df_T =keepValidPositions(df_T,fvp,lvp) #keep valid positions of experiment


    return df_T, strain, repetition, lvp


def keepValidPositions(dfE, fvp, lvp):
    # Function that takes the dataframe and keeps only the valid positions

    # look for the value of the positions in the dataframe
    posiciones = dfE.Position.unique()

    # Find the index where the positions are valid
    idx = np.where((posiciones >= fvp) & (posiciones <= lvp))

    #save valid positions
    posVal = posiciones[idx]


    dfE_val = dfE[dfE.Position.isin(posVal)].copy() #get new dataframe only with valid positions

    return dfE_val



import pandas as pd
import math
import matplotlib.pyplot as plt
import scipy
import os



def dgumbel(x,mu,s): #density function
    return math.exp((mu - x)/s - math.exp((mu - x)/s))/s

def pgumbel(q,mu,s): #distribution function
    return math.exp(-(math.exp(-((q - mu)/s))))

def qgumbel(p,mu,s): #Quantile function
    return (mu - s) * math.log(-math.log(p))

def gumbelPlot(df,strain,repetition):
    dataframe,positions= extractMaximums(df)
    outlieres =gumbelAdjust(dataframe,positions,strain,repetition)
    return outlieres

def extractMaximums(df):
    df_positions =df.Position.unique() #get positions
    entropies = []
    for position in df_positions:
        entropy=df.loc[df.Position==position,'Entropy'].max() #maximum entropy
        entropies.append(entropy)

    df = pd.DataFrame(entropies, columns=['Entropy']) #create df with entropies maxiumums
    return df,df_positions


def get_hist(ax):
    n,bins = [],[]
    for rect in ax.patches:
        ((x0, y0), (x1, y1)) = rect.get_bbox().get_points()
        n.append(y1-y0)
        bins.append(x0) # left edge of each bin
    bins.append(x1) # also get right edge of last bin

    return n,bins




def gumbelAdjust(dfMax,positions,strain,repetition):
    pathPip=str(os.path.dirname(pd.__file__)).split('pandas')[0]
    sub = dfMax.quantile(.95, axis = 0)
    quantile= sub.iloc[0] #get quantile number
    listDF = dfMax.values.tolist()
    subsample=[]
    subpos = []
    index = 0
    for value in listDF: #every value in dfMax
        if value <= quantile: #if not in left tail
            subsample.append(value)
            subpos.append(positions[index])
        index +=1

    mu, sigma = scipy.stats.gumbel_r.fit(subsample) #fit subsample with gumbel

    #points of interest
    distribution = []
    dfMax['Position'] = positions

    for p in positions:
        distribution.append(1-pgumbel(dfMax.loc[dfMax.Position==p,'Entropy'].values.tolist(),mu,sigma)) #calcule p-value and add to distribution list
    dfDist = pd.DataFrame(distribution,columns=['dist']) #create DataFrame
    dfDist['positions'] = positions
    df =dfDist.nsmallest(10,'dist') #10 smallest p-value
    outliers = df['positions'].values.tolist()
    i = 0
    auxlen = len(outliers)
    pos = 'Positions of interest: ' #create string with every outlier
    while i < auxlen:
        if (i+1) == auxlen: #last position
            pos += str(outliers[i])
        else:
            pos += str(outliers[i]) + ', '
        i +=1



    dfHist = pd.DataFrame(subsample,columns=['Entropy'])
    dfAux = pd.DataFrame(subsample,columns=['Entropy'])
    axsub=dfHist['Entropy'].hist(bins=20,grid=False, xlabelsize=12, ylabelsize=12, density=True) #create histogram
    n, bins = get_hist(axsub) #get points from histogram


    plt.xlabel("Entropy ", fontsize=15)
    plt.ylabel("Frequency", fontsize=15)
    plt.xlim([0.0, 0.2])

    dfAux['Position']=subpos

    x=[]

    y=[]
    for point in bins: #x,y from gumble
        x.append(point)
        a=dgumbel(point,mu,sigma)
        y.append(a)
        #plt.plot(a,x,'bo')
        #aux=aux+sum



    plt.plot(x,y, 'r-', label='density gumbel', linewidth=1) #plot gumble
    name = pathPip+'src/plot_images/Gumbel'+'_'+strain+'_'+repetition + '.png'
    plt.savefig(name)
    plt.close()
    return pos #return positions of interest






import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from sklearn.manifold import TSNE
from src.Format_DF import totalExperimentsToPassages, totalExperimentsToPositions, formatStrains


def tsnePassages(experiments, perp=None, lr=None, df_results=None): #t-SNE for passages

    #calculate t-SNE passages normal with this combinations or if user selected perp and lr
    if ((perp==None) and (lr==None)):
        perps = [3, 6, 12, 15]
        learning_r = [10, 100, 300, 500]
    elif ((perp!=None) and (lr==None)):
        perps=perp
        learning_r = [10, 100, 300, 500]
    elif ((perp==None) and (lr!=None)):
        perps = [3, 6, 12, 15]
        learning_r=lr
    else:
        perps=perp
        learning_r=lr

    data, infos, strains = totalExperimentsToPassages(experiments) #transform experiments
    cant, df_Pas_Strains = formatStrains(strains) #format strains
    df_infos=pd.DataFrame(infos, columns=["Info"]) #create dataframe with info


    data_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "Strain", "StrainNumber", "Info", "Perp", "LR"]) #create new dataframe

    data_Tot=pd.DataFrame()

    #calculate every transformation with perp y lerning rate combinations and save in dataframe
    for p in perps:
        for lr in learning_r:
            transformedData = TSNE(perplexity=p, learning_rate=lr, random_state=111).fit_transform(data)
            data_Aux["tsne_1"]=transformedData[:,0]
            data_Aux["tsne_2"]=transformedData[:,1]
            data_Aux["Strain"]=df_Pas_Strains["Strain"]
            data_Aux["StrainNumber"]=df_Pas_Strains["StrainNumber"]
            data_Aux["Info"]=df_infos["Info"]
            data_Aux.loc[:, "Perp"]=p
            data_Aux.loc[:, "LR"]=lr
            data_Aux.loc[:, "Plot"]=0
            data_Tot=data_Tot.append(data_Aux, ignore_index=True)



    if df_results is None: #if normal execution(not perp and lr added by user)
        df_results=data_Tot
    else:
        df_results=df_results.append(data_Tot, ignore_index=True) #add lr and perp to old info

    return df_results #return dataframe

def tsnePositions(experiments, perp=None, lr=None, df_results=None):
    #same as tsnePassages, but with positions. Combinations are less than passages due  the number of points
    if ((perp==None) and (lr==None)):
        perps = [3, 12]
        learning_r = [10, 300]
    elif ((perp!=None) and (lr==None)):
        perps=perp
        learning_r = [10, 100, 300, 500]
    elif ((perp==None) and (lr!=None)):
        perps = [3, 6, 12, 15]
        learning_r=lr
    else:
        perps=perp
        learning_r=lr

    data, infos, strains = totalExperimentsToPositions(experiments)
    cant, df_Pas_Strains = formatStrains(strains)
    df_infos=pd.DataFrame(infos, columns=["Info"])


    data_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "Strain", "StrainNumber", "Info", "Perp", "LR"])

    data_Tot=pd.DataFrame()


    for p in perps:
        for lr in learning_r:
            transformedData = TSNE(perplexity=p, learning_rate=lr, random_state=111).fit_transform(data)
            data_Aux["tsne_1"]=transformedData[:,0]
            data_Aux["tsne_2"]=transformedData[:,1]
            data_Aux["Strain"]=df_Pas_Strains["Strain"]
            data_Aux["StrainNumber"]=df_Pas_Strains["StrainNumber"]
            data_Aux["Info"]=df_infos["Info"]
            data_Aux.loc[:, "Perp"]=p
            data_Aux.loc[:, "LR"]=lr
            data_Aux.loc[:, "Plot"]=0
            data_Tot=data_Tot.append(data_Aux, ignore_index=True)



    if df_results is None:
        df_results=data_Tot
    else:
        df_results=df_results.append(data_Tot, ignore_index=True)

    return df_results



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import LinearLocator, FormatStrFormatter



def getEntropySurface(dataFrame,strain,repetition):
    pathPip = str(os.path.dirname(np.__file__)).split('numpy')[0]
    positions = np.array(dataFrame['Position'].astype(int))  # get positions
    passages = np.array(dataFrame['Passage'].astype(int))  # get passages
    entropy = np.array(dataFrame['Entropy'])  # get entropy

    cols = np.unique(positions).shape[0]  # get unique positions

    y = positions.reshape(-1, cols)  # reshape 1-D to dimension 2-D with passages,positions
    x = passages.reshape(-1, cols)  # reshape 1-D to dimension 2-D with passages,positions
    z = entropy.reshape(-1, cols)  # reshape 1-D to dimension 2-D with passages,positions

    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.grid(True)

    surf = ax.contour3D(x, y, z, 100, cmap='twilight_shifted')  # plot surface

    ax.zaxis.set_major_locator(LinearLocator(10))
    ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))

    # Add a color bar which maps values to colors.
    fig.colorbar(surf, shrink=0.5, aspect=5)  # colorbar

    # labls
    ax.set_xlabel('Passages')
    ax.set_ylabel('Positions')
    ax.set_zlabel('Entropy')
    # save image
    name =pathPip+ 'src/plot_images/entropySurface_' +strain+'_' +repetition+ '.png'
    plt.savefig(name)
    plt.close()





from PyQt5 import QtWidgets,uic
from src.dialogError import dError
import os

class valuesPositions(QtWidgets.QDialog):
    def __init__(self, parent=None):
        super(valuesPositions, self).__init__()
        pathPip = str(os.path.dirname(uic.__file__)).split('PyQt5')[0]
        path_buttons = pathPip + 'src/buttons_and_background/'
        uic.loadUi(pathPip+'src/ui_files/dialogPositions.ui', self)
        self.pushButton.clicked.connect(self.setParameters)
        self.fvpValue = 774
        self.lvpValue = 7332
        self.setIt = False

    def setParameters(self):

        fvp = self.FVP.toPlainText()
        lvp = self.LVP.toPlainText()
        try:
            if fvp == '':
                fvp = 0
            if lvp == '':
                lvp = 0

            self.fvpValue = int(fvp)
            self.lvpValue = int(lvp)
            if self.fvpValue > self.lvpValue:
                error_dialog = dError('LVP must be Higher tan FVP')
                error_dialog.exec()
            else:
                self.setIt = True
                self.close()
        except:
            error_dialog = dError('Please enter only integer values')
            error_dialog.exec()()
            self.close()

    def getIfSet(self):
        return self.setIt
    def getFVP(self):
        return self.fvpValue
    def getLVP(self):
        return self.lvpValue


import numpy as np
import pandas as pd
from PIL import Image
from sklearn.decomposition import PCA
import matplotlib
import os
matplotlib.use('Qt5Agg')
from PyQt5 import QtCore, QtWidgets, uic
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from src.PlotEntropyPosition import  plotEntropiesPosition




class FigurePCA(FigureCanvas):

    def __init__(self, dfPointsPCA, pcaVector, infoDF, nameExps,dfs, parent,allinfo=None, width=5, height=4, dpi=100): #create pca figure

        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        FigureCanvas.__init__(self, fig)
        self.parent = parent

        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

        self.dfPointsPCA = dfPointsPCA
        self.infoDF = infoDF
        self.pcaVector = pcaVector
        self.nameExps = nameExps
        self.dfsAux = dfs
        self.fig = fig
        self.aux = []
        if allinfo is not None: #if all information is in this df(because is imported)
            self.allInfoDF=allinfo
            self.reWritePCAInfoImported() #rewrite information to obtain the same information as if it were not imported
            self.isImported = True
        else:
            self.allInfoDF = self.joinDataFrames() #create dataframe with all info. This is for export function
            self.isImported = False

        self.compute_initial_figure() #plot figure
        self.name = ''
        self._flag = False

    def joinDataFrames(self):

        df2 = pd.DataFrame(self.pcaVector) #create dataframe from numpy pcaVector
        df3 = pd.DataFrame(self.infoDF) #create new dataframe from info arry
        df = pd.concat([self.dfPointsPCA,df2,df3]) #concat all info
        df = df.reset_index() #reset index

        return df

    def updatePCA(self,dfPointsPCA,pcaVector,infoDF,nameExps,dfs,allinfo=None): #update figure, new PCA
        self.dfPointsPCA = dfPointsPCA
        self.pcaVector = pcaVector
        self.infoDF = infoDF
        self.nameExps = nameExps
        self.dfsAux = dfs
        if allinfo is not None:
            self.allInfoDF=allinfo
            self.reWritePCAInfoImported()
            self.isImported = True
        else:
            self.isImported = False
            self.allInfoDF = self.joinDataFrames()

        self.axes.cla() #clear plot
        self.compute_initial_figure() #plot figure

    def reWritePCAInfoImported(self):
        count_row = self.allInfoDF.shape[0] #counts rows
        index = int((((count_row - 2) / 2)) * -1) #get index(there are two rows for pcaVector, and there are two rows for every experiment(info and pca points))
        indexT = index - 2
        df = self.allInfoDF.iloc[indexT:] #get last indexT rows(has info and pcaVector)
        df = df.drop(['index','PCA_1','PCA_2','Repetition','Strain'],axis=1)
        df = df.loc[:, ~df.columns.str.contains('^Unnamed')]
        dfI = df.iloc[index:] #get last index rows(has info)
        dfvector = df.head(2) #get only pcaVector, there is in first 2 rows
        vectorPCA = dfvector.to_numpy()
        info = dfI.to_numpy()
        self.infoDF=info
        vectorPCA = vectorPCA.astype(np.float)
        self.pcaVector=vectorPCA
        df = self.allInfoDF[['PCA_1','PCA_2','Strain','Repetition']] #get all info again, and get only this columns
        df = df.dropna()  #drops na, drops every row with nulls
        self.dfPointsPCA=df


    def compute_initial_figure(self):

        strains = self.dfPointsPCA.Strain.unique()
        self.fig.canvas.mpl_connect('button_press_event', self.check_click) #connect with function to handle click evets
        self.aux = self.axes.scatter(self.pcaVector[0, :], self.pcaVector[1, :], marker='o', c="grey", alpha=0.2) #plot eigenvector
        #to scale points, gets max in eigenvector and eigenvalues
        pointx = np.max(self.pcaVector[0, :])
        pointy = np.max(self.pcaVector[1, :])
        auxPointX = np.max(self.dfPointsPCA["PCA_1"].values)
        auxPointY = np.max(self.dfPointsPCA["PCA_2"].values)
        scalex = auxPointX / pointx
        scaley = auxPointY / pointy
        for s in strains: #plot eigenvalues
            d = self.dfPointsPCA.loc[self.dfPointsPCA['Strain'] == s]
            x = d.PCA_1 / scalex
            y = d.PCA_2 / scaley
            self.axes.scatter(x, y, label=s)
        self.axes.set_xlabel('X1')
        self.axes.set_ylabel('X2')
        self.axes.legend()
        title = 'Distribution coefficients of principal components position '
        plt.suptitle(title, fontsize=16)
        self.axes.grid(True)
        self.draw()

    def find_nearest(self, points, value): #find nearest value in eigenvector for value data clicked. Returns index position
        arrayAux = np.asarray(points)
        idx = (np.abs(arrayAux - value)).argmin()
        return idx

    def check_click(self, event): #checks if clicks position have a plot point
        if event.inaxes == self.axes:
            cont, ind = self.aux.contains(event)
            if cont and not self.isImported:
                self.onclick(event)


    def onclick(self, event):
        pathPip = str(os.path.dirname(Image.__file__)).split('PIL')[0]
        ix, iy = event.xdata, event.ydata
        indx = self.find_nearest(self.pcaVector[0,], ix)
        exp = self.infoDF[0,]
        experiment = exp[indx]
        experimentAux = experiment.split(',')
        position = experimentAux[3]
        plotEntropiesPosition(int(position), self.dfsAux)
        self.name = pathPip+'src/plot_images/Entropy_' + position + '.png'
        img = Image.open(self.name)
        titleAux = pathPip+"plot_images/Entropy for position " + position
        img.show(title=titleAux)

    def exportPCA(self,filename): #export PCA result
        if self.isImported:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Unable to export an imported file')
            error_dialog.exec()
        else:
            self.allInfoDF.to_csv(filename)
            return self.nameExps




from PyQt5 import QtWidgets,uic, QtCore
from PyQt5.QtGui import QPixmap
import os

class dError(QtWidgets.QDialog):
    def __init__(self, errorMessage,parent=None):
        super(dError, self).__init__()
        pathPip = str(os.path.dirname(uic.__file__)).split('PyQt5')[0]
        uic.loadUi(pathPip+'src/ui_files/dialogError.ui', self)
        self._flag = False
        self.pushButton.clicked.connect(self.closeWindow)
        self.pushButton.setToolTip("Click to close window")
        self.errorText.setText(errorMessage)



    def closeWindow(self):
        self.close()



from PyQt5 import QtWidgets, uic, QtCore
from PyQt5.QtWidgets import QFileDialog, QLabel, QSlider
import sys
from src.ReadData import readData,keepValidPositions
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QPixmap, QPalette, QBrush, QImage, QIcon, QMovie
from src.pcaClass import FigurePCA
from src.dialogPositions import valuesPositions
import pandas as pd
from src.dialogInterpolation import dInterpolation
from src.threadsOpenClass import LoadFile, LoadFolder
from src.threadsPlotClass import plotSurface,plotPositionEntropy,plotGumbel, codonDecay
from src.tsneClass import FigureTSNE
from src.tsneThreadProcess import tsneProcess,tsneProcessUpdate
from src.lvrvClass import FigureLVRV
from src.lvrvThread import lvrv_poly,lvrv_average
from src.codonDecayClass import FigureCD
from src.threadPCA import pcaProcess
from datetime import datetime
import logging
import pymongo
import socket
import os
import json
from jsonschema import validate
import csv

def dbconnect(mode):
    if mode:
        client = pymongo.MongoClient("mongodb+srv://vevdb:MDp1atkxAv9R2KxZ@cluster0.lu8xi.gcp.mongodb.net/")
        db = client['VEV']
        return db
    else:
        return None

def logger(new,experiment,log):
    logging.info(log)

def getExperiments(ne,nameExps):
    #299_1(774-7332)
    otherexperiments = ''
    for exp in nameExps:
        if ne != exp:
            experiment = exp.split('.')[0]
            otherexperiments =otherexperiments + experiment + ' '
    return otherexperiments


def initLoggin():
    pathPip = str(os.path.dirname(pymongo.__file__)).split('pymongo')[0]
    date_time = datetime.now().strftime("%d-%m-%Y")
    logging.basicConfig(filename=pathPip+'src/logs/vev-'+date_time+'.log',format='%(asctime)s %(message)s', level=logging.INFO)
    logging.info('VEV Started')

class Interface(QtWidgets.QMainWindow):
    def __init__(self, mode=None):
        super(Interface, self).__init__()
        pathPip = str(os.path.dirname(pymongo.__file__)).split('pymongo')[0]
        uic.loadUi(pathPip+'src/ui_files/mainwindow.ui', self)
        self._flag = False
        if mode==None:
            self.Online= True
        else:
            self.Online = False
        #paths
        self.path_buttons = pathPip + 'src/buttons_and_background/'
        self.path_pkl = pathPip + 'src/data_experiments/'
        self.path_grafics = pathPip + 'src/plot_images'
        self.path_schemas = pathPip + 'src/'
        self.loaded_dfs = []
        #################
        self.threadpool = QtCore.QThreadPool()  # handle threads
        self.label_lvrv.hide()
        self.input_lvrv.hide()
        self.slidersTSNE = QtWidgets.QVBoxLayout(self.frame_sliders)
        self.arraySliders = []
        initLoggin()
        self.db = dbconnect(self.Online)
        self.namePC = socket.gethostname()
        self.addUser()
        # call functions

        self.buttonLoadExperiment.clicked.connect(self.loadExperiment)
        self.buttonLoadParquet.clicked.connect(self.loadExperimentParquet)
        self.plotPositionButton.clicked.connect(self.plotPostionFunction)
        self.plotSurfaceButton.clicked.connect(self.plotSurfaceFunction)
        self.codonButton.clicked.connect(self.codonDecayFunction)
        self.PCAButton.clicked.connect(self.pcaFunction)
        self.tsneButton.clicked.connect(self.tsneFunctions)
        self.buttonInterpolation.clicked.connect(self.interpolateFunction)
        self.gumbelButton.clicked.connect(self.gumbelWidget)
        self.addPerpLRButton.clicked.connect(self.tsneAddPerpAndLR)
        self.mvAverage.toggled.connect(lambda: self.changeLabel(self.mvAverage))
        self.poly.toggled.connect(lambda: self.changeLabel(self.poly))
        self.lvrv_button.clicked.connect(self.lvrvFunction)
        self.LRButton.clicked.connect(self.getLastResults)
        self.ClusterPlotButton.clicked.connect(self.clusteringFunction)
        self.exportCluster.clicked.connect(self.exportClusterDF)

        # exports functions
        self.exportCSVTSNE.clicked.connect(self.exportTSNE)
        self.importCSVTSNE.clicked.connect(self.importTSNE)
        self.exportCSVLVRV.clicked.connect(self.exportLVRV)
        self.ImportCSVLVRV.clicked.connect(self.importLVRV)
        self.ExportCSVPCA.clicked.connect(self.exportPCA)
        self.ImportCSVPCA.clicked.connect(self.importPCA)

        # Move stacked widgets

        self.stackedWidget.setCurrentIndex(0)  # main
        self.buttonplotposition.clicked.connect(self.entropyPosition)
        self.buttonEntropySurface.clicked.connect(self.entropySurface)
        self.buttonCodonDecay.clicked.connect(self.codonDecayWidget)
        self.buttonPCAFunction.clicked.connect(self.pcaWidget)
        self.buttonTsne.clicked.connect(self.tsne)
        self.plotGumbelButton.clicked.connect(self.gumbelFunction)
        self.buttonLVRV.clicked.connect(self.LVRV)
        self.buttonClustering.clicked.connect(self.Clustering)

        # style
        self.setMinimumSize(1000, 568)
        self.setWindowIcon(QIcon(self.path_buttons + 'ImageICON.jpg'))
        self.buttonLoadExperiment.setStyleSheet(
            "QPushButton{border-image:url(" + self.path_buttons + "Load Folder.png);}")
        self.buttonLoadExperiment.setToolTip("Click this button to open experiment folder")
        self.buttonLoadParquet.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "Load File.png);}")
        self.buttonLoadParquet.setToolTip("Click this button to open experiment file")
        self.buttonLVRV.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "LVRV.png);}")
        self.buttonLVRV.setToolTip("Plot Leading Variations and Random Variations")
        self.buttonplotposition.setStyleSheet(
            "QPushButton{border-image:url(" + self.path_buttons + "Plot Entropy x Position.png);}")
        self.buttonplotposition.setToolTip("Plot Entropy times position")
        self.buttonEntropySurface.setStyleSheet(
            "QPushButton{border-image:url(" + self.path_buttons + "Plot Entropy Surface.png);}")
        self.buttonEntropySurface.setToolTip("Plot Entropy Surface of an experiment")
        self.buttonCodonDecay.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "Codon Decay.png);}")
        self.buttonCodonDecay.setToolTip("Plot Codon Decay of an experiment")
        self.buttonPCAFunction.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "PCA.png);}")
        self.buttonPCAFunction.setToolTip("Plot PCA")
        self.gumbelButton.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "Gumbel.png);}")
        self.gumbelButton.setToolTip("Plot Gumbel distribution")
        self.buttonTsne.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "t-SNE.png);}")
        self.buttonTsne.setToolTip("Enter to t-SNE plot options")
        self.buttonInterpolation.setStyleSheet(
            "QPushButton{border-image:url(" + self.path_buttons + "Interpolation.png);}")
        self.buttonInterpolation.setToolTip("Interpolate experiment data")
        self.LRButton.setToolTip("Export info of results executed order by last modified")
        self.buttonClustering.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "Clustering.png);}")
        self.buttonClustering.setToolTip("Clustering Analysis ")

        # home buttons
        self.home1.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home1.setToolTip("Return to home")
        self.home2.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home2.setToolTip("Return to home")
        self.home3.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home3.setToolTip("Return to home")
        self.home4.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home4.setToolTip("Return to home")
        self.home5.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home5.setToolTip("Return to home")
        self.home6.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home6.setToolTip("Return to home")
        self.home7.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home7.setToolTip("Return to home")
        self.home8.setStyleSheet("QPushButton{border-image:url(" + self.path_buttons + "home.png);}")
        self.home8.setToolTip("Return to home")
        # connect home buttons
        self.home1.clicked.connect(self.returnHome)
        self.home2.clicked.connect(self.returnHome)
        self.home3.clicked.connect(self.returnHome)
        self.home4.clicked.connect(self.returnHome)
        self.home5.clicked.connect(self.returnHome)
        self.home6.clicked.connect(self.returnHome)
        self.home7.clicked.connect(self.returnHome)
        self.home8.clicked.connect(self.returnHome)

        # TABLE VIEW MODELS##
        self.model = QStandardItemModel()
        self.model.setHorizontalHeaderLabels(['Experiment '])
        self.tableView_PlotPostion.setModel(self.model)
        self.tableView_PlotPostion.horizontalHeader().setSectionResizeMode(1)
        self.tableTSNE.setModel(self.model)
        self.tableTSNE.horizontalHeader().setSectionResizeMode(1)
        self.tablePCA.setModel(self.model)
        self.tablePCA.horizontalHeader().setSectionResizeMode(1)
        self.tableViewCodon.setModel(self.model)
        self.tableViewCodon.horizontalHeader().setSectionResizeMode(1)
        self.tableViewEntropySurface.setModel(self.model)
        self.tableViewEntropySurface.horizontalHeader().setSectionResizeMode(1)
        self.tableViewGumbel.setModel(self.model)
        self.tableViewGumbel.horizontalHeader().setSectionResizeMode(1)
        self.tableLVRV.setModel(self.model)
        self.tableLVRV.horizontalHeader().setSectionResizeMode(1)
        self.tableCluster.setModel(self.model)
        self.tableCluster.horizontalHeader().setSectionResizeMode(1)
        # Sliders
        self.perplexitySlider.setMaximum(3)
        self.perplexitySlider.setMinimum(0)
        self.lrSlider.setMaximum(3)
        self.lrSlider.setMinimum(0)
        self.clusterSlider.setMaximum(2)
        self.clusterSlider.setMinimum(0)
        self.typeCluster.setMaximum(1)
        self.typeCluster.setMinimum(0)
        self.clusterSlider.valueChanged.connect(self.updateCluster)
        self.typeCluster.valueChanged.connect(self.updateTypeCluster)
        self.perplexitySlider.valueChanged.connect(self.updateTSNE)
        self.lrSlider.valueChanged.connect(self.updateTSNE)
        # FIGURES
        self.pca = None
        self.tsne = None
        self.lvrv = None
        self.condonDecay = None
        self.cluster = ''
        self.clusterDF = None
        # AUXILIAR
        self.perAux = [3, 6, 12, 15]
        self.lrAux = [10, 100, 300, 500]
        self.numberCluster = [2, 4, 6]
        self.movie = QMovie(self.path_buttons + 'Loading.gif')

    def scale_image(self, size):  # scale background image
        imageBack = QImage(self.path_buttons + 'background.png')
        palette = QPalette()
        palette.setBrush(QPalette.Window, QBrush(imageBack))
        self.setPalette(palette)

    def resizeEvent(self, e):  # function to handle resize main window
        if not self._flag:
            self._flag = True
            self.scale_image(e.size())
            QtCore.QTimer.singleShot(30, lambda: setattr(self, "_flag", False))
        super().resizeEvent(e)

        # CHANGE LABEL -- function to handle label in LV-RV function(Moving average or Polynomial)

    def changeLabel(self, b):  # b is the radio button
        self.label_lvrv.show()
        self.input_lvrv.show()
        if b.text() == "Moving Average":
            if b.isChecked() == True:
                self.label_lvrv.setText("Select Average Window:")
            else:
                self.label_lvrv.setText("Select Polynomial Grade:")

        else:
            if b.isChecked() == True:
                self.label_lvrv.setText("Select Polynomial Grade:")
            else:
                self.label_lvrv.setText("Select Average Window:")
        # Load FILES AND FOLDERS

    def loadExperiment(self):
        # opens dialog for user to select folder experiment
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName = QFileDialog.getExistingDirectory(self, directory=os.getcwd(), caption='Select a directory',
                                                    options=options)
        if fileName != "":  # If user select folder
            vp = valuesPositions()
            vp.exec()
            fvp = vp.getFVP()
            lvp = vp.getLVP()
            setIt = vp.getIfSet()
            if setIt:  # if user set valid positions(fvp,lvp) then execute thread for load folder
                thread = LoadFolder(fileName, fvp, lvp)
                thread.signals.error.connect(self.openError)  # linked function to main thread if thread executes fails
                thread.signals.finished.connect(
                    self.openOK)  # linked function to main thread if thread executes correctly
                self.threadpool.start(thread)
            else:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Parameters was not correctly selected')
                error_dialog.exec()

    def loadExperimentParquet(self):
        # opens dialog for user to select  experiment parquet file
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        (fileName, aux) = QFileDialog.getOpenFileName(self, directory=self.path_pkl, caption='Select a .parquet File',
                                                      filter='*.parquet', options=options)
        if fileName != "":  # If user select file then execute thread for load experiement file
            thread = LoadFile(fileName)
            thread.signals.error.connect(self.openError)  # linked function to main thread if thread throws an error
            thread.signals.finished.connect(self.openOK)  # linked function to main thread if thread executes correctly
            self.threadpool.start(thread)

        # Complete or Errors with open

    def openOK(self, parameters):  # parameters emitted by the executed thread
        strain = parameters[0]
        repetition = parameters[1]
        fvp = parameters[2]
        lvp = parameters[3]
        control = parameters[4]
        if (strain, repetition, fvp, lvp) in self.loaded_dfs:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('This experiment is already loaded')
            error_dialog.exec()
        else:
            self.loaded_dfs.append((strain, repetition, fvp, lvp))  # add to currents loaded experiments
            experimentName = "Strain: " + strain + " " + "Repetition: " + repetition
            row = (QStandardItem(experimentName))
            self.model.appendRow(row)  # add experiment to the table of experiments
            self.refreshFunction()  # refresh experiments tables
            if control:
                nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
                log = 'Experiment ' + nameLog + 'was loaded and entropy was calculated with First Value Position(FVP) = ' + str(
                    fvp) + ' and Last Value Position(LVP) = ' + str(lvp)
                logger(True, nameLog, log)  # logging function
                de = QtWidgets.QMessageBox()
                de.setText('Folder: ' + experimentName + ' loaded')
                de.exec()  # dialog to let the user know that the experiment was loaded succesufully
                self.addExperiment(strain, repetition, fvp, lvp)
            else:

                logging.info(' Experiment already calculated was load: ' + strain + '_' + repetition + '(' + str(
                    fvp) + '-' + str(lvp) + ')')

    def openError(self, error):  # if load throws error
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage(
            'An error has occurred when an experiment was loaded. Please verify if the folder contains only csv experiments files')
        error_dialog.exec()
        logging.info('An error has occurred when an experiment was loaded')

    def refreshFunction(self):  # refhresh tables
        self.tablePCA.update()
        self.tableViewEntropySurface.update()
        self.tableView_PlotPostion.update()
        self.tableViewCodon.update()
        self.tableTSNE.update()
        self.tableViewGumbel.update()
        self.tableLVRV.update()
        self.tableCluster.update()

    def interpolateFunction(self):  # function for interpolation
        di = dInterpolation(self.loaded_dfs)  # create interpolation class
        di.exec()  # execute interpolation
        executed, typeInterpolation, strain, repetition, fvp, lvp, passage = di.getInfo()  # get info from interpolation executed
        if executed:
            nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
            log = typeInterpolation + ' interpolation was made for passage number ' + passage
            logger(False, nameLog, log)
            self.addEntrydb(strain, repetition, fvp, lvp, 'Interpolation ' + typeInterpolation, passage, '')
            self.addFunctions('Interpolation ' + typeInterpolation, passage)

        # PLOTS AND Interactive Functions

    def tsneAddPerpAndLR(self):  # Function to handle user add new Perplexity and LR.
        try:
            perp = int(self.newPerplexity.toPlainText())  # value of perp
            lr = int(self.newLR.toPlainText())  # value of LR
            if self.tsne is None:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please plot t-SNE method first')
                error_dialog.exec()
            else:
                dfs, data, nameExps = self.tsne.getData()  # get tsne data ploted
                auxType = self.tsne.getType()  # get t-SNE type
                if auxType == 'Passages':
                    thread = tsneProcessUpdate(dfs, perp, lr, data, "Passages", nameExps)
                else:
                    thread = tsneProcessUpdate(dfs, perp, lr, data, "Positions", nameExps)
                thread.signals.error.connect(self.errorTSNEProcess)
                thread.signals.finished.connect(self.okProcessTSNEUpdate)

                self.loading.setMovie(self.movie)  # for loading bar
                self.movie.start()  # start gif
                self.threadpool.start(thread)  # start thread

        except:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please enter integer values')
            error_dialog.exec()

    def okProcessTSNEUpdate(self, parameters):
        self.movie.stop()  # stop gif
        self.loading.clear()  # clear loading bar
        data = parameters[0]
        perp = parameters[1]
        lr = parameters[2]
        typeTSNE = parameters[3]
        nameExps = parameters[4]

        self.tsne.updateData(data, perp, lr, nameExps)  # update LR and Perp with user values
        for ne in nameExps:  # get names of experiments plotted in t-SNE
            otherexperiments = getExperiments(ne, nameExps)
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            log = typeTSNE + ' t-SNE executed with perplexity = ' + str(perp) + ' and learning rate = ' + str(
                lr) + ' with this experiments: ' + otherexperiments
            # logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp,
                            't-SNE ' + typeTSNE + ' with perp=' + str(perp) + ' and lr=' + str(lr), otherexperiments,
                            '')
            self.addFunctions('t-SNE ' + typeTSNE + ' with perp=' + str(perp) + ' and lr=' + str(lr), otherexperiments)

    def clearLayout(self, layout):  # clear ON/OFF sliders in t-SNE for new t-SNE execution
        self.arraySliders = []
        if layout:
            while layout.count():
                item = layout.takeAt(0)
                widget = item.widget()
                if widget:
                    widget.deleteLater()
                else:
                    self.clearLayout(item.layout())
                layout.removeItem(item)

    def addSlidersTSNE(self, strains):  # add ON/OFF slider for t-SNE
        self.clearLayout(self.slidersTSNE)

        for strain in strains:
            label = QLabel()
            label.setText(str(strain))
            self.slidersTSNE.addWidget(label)
            slid = QSlider(QtCore.Qt.Horizontal)
            slid.setMaximum(1)
            slid.setMinimum(0)
            self.slidersTSNE.addWidget(slid)
            slid.setValue(0)
            slid.valueChanged.connect(self.updateStrainsTSNE)
            self.arraySliders.append([slid, strain])

    def tsneFunctions(self):  # new t-SNE execution
        dfsAux = []
        dfs = []
        strains = []
        nameExperiments = []
        indices = self.tableTSNE.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()


        elif (self.rb_passages.isChecked() == False and self.rb_positions.isChecked() == False):
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select method type')
            error_dialog.exec()

        else:  # valid parameters
            try:
                fvpMayor = 0
                lvpMenor = 0
                for index in sorted(indices):  # selected experiments in table
                    experimentIndex = (index.row())
                    df = self.loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    if strain not in strains:  # unique strains for ON/OFF strains
                        strains.append(strain)
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    if fvpMayor == 0 and lvpMenor == 0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:  # different experiments positions(fvp,lvp)
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(self.path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExperiments.append(experimentNametxt)  # array of experiments names
                for dataFrame in dfsAux:
                    df = keepValidPositions(dataFrame, fvpMayor, lvpMenor)
                    dfs.append(df)  # array of dataframes

                self.addSlidersTSNE(strains)  # add on off sliders
                if self.rb_passages.isChecked() == True:  # tpye of t-SNE
                    auxType = 'Passages'
                else:
                    auxType = 'Positions'
                thread = tsneProcess(dfs, auxType, nameExperiments)  # Class QRunnable tsneProcess
                thread.signals.error.connect(self.errorTSNEProcess)
                thread.signals.finished.connect(self.okProcessTSNE)
                self.loading.setMovie(self.movie)  # loading bar
                self.movie.start()
                self.threadpool.start(thread)  # execute thread
            except:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error')
                error_dialog.exec()

    def errorTSNEProcess(self):
        self.movie.stop()
        self.loading.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated t-SNE')
        error_dialog.exec()
        logging.info('An error has occurred when calculated t-SNE')

    def okProcessTSNE(self, parameters):  # parameters emitted for t-SNE
        self.newPerplexity.clear()
        self.newLR.clear()
        self.movie.stop()
        self.loading.clear()
        dfs = parameters[0]
        data = parameters[1]
        tsneType = parameters[2]
        nameExp = parameters[3]
        if self.tsne == None:  # first execution
            boxAux = QtWidgets.QVBoxLayout(self.tsneFrame)
            self.tsne = FigureTSNE(dfs, data, tsneType, nameExp, self.tsneFrame, width=6, height=5,
                                   dpi=100)  # create figure TSNE for plot and handle mouse events
            boxAux.addWidget(self.tsne)
        else:
            # handle sliders to set in first positions
            self.perplexitySlider.valueChanged.disconnect()
            self.lrSlider.valueChanged.disconnect()
            self.tsne.newtSNE(dfs, data, tsneType, nameExp)
            self.perplexitySlider.setValue(0)
            self.lrSlider.setValue(0)
            self.perplexity.setText(str(3))
            self.lr.setText(str(10))
            self.perplexitySlider.valueChanged.connect(self.updateTSNE)
            self.lrSlider.valueChanged.connect(self.updateTSNE)
        if tsneType == 'Passages':  # sliders has 4 positions
            self.perplexitySlider.setMaximum(3)
            self.lrSlider.setMaximum(3)
            self.perAux = [3, 6, 12, 15]
            self.lrAux = [10, 100, 300, 500]
        else:  # sliders has 2 positions
            self.perplexitySlider.setMaximum(1)
            self.lrSlider.setMaximum(1)
            self.perAux = [3, 12]
            self.lrAux = [10, 300]
        for ne in nameExp:
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            otherexperiments = getExperiments(ne, nameExp)
            log = tsneType + ' t-SNE executed with this experiments: ' + otherexperiments
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 't-SNE ' + tsneType, otherexperiments, '')
            self.addFunctions('t-SNE ' + tsneType, otherexperiments)

    def updateTSNE(self):  # handle sliders move
        perplexity = self.perAux[self.perplexitySlider.value()]
        lr = self.lrAux[self.lrSlider.value()]
        self.perplexity.setText(str(perplexity))
        self.lr.setText(str(lr))
        if self.tsne == None:  # not t-SNE executed
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this, plot t-SNE method')
            error_dialog.exec()
            self.perplexitySlider.valueChanged.disconnect()
            self.lrSlider.valueChanged.disconnect()
            self.perplexitySlider.setValue(0)
            self.lrSlider.setValue(0)
            self.perplexity.setText(str(3))
            self.lr.setText(str(10))
            self.perplexitySlider.valueChanged.connect(self.updateTSNE)
            self.lrSlider.valueChanged.connect(self.updateTSNE)
        else:
            self.tsne.updateParameters(perplexity, lr)  # update figure

    def getChanges(self):  # changes in on off sliders
        changes = []
        for [slider, strain] in self.arraySliders:
            if slider.value() == 1:
                changes.append(strain)
        return changes

    def updateStrainsTSNE(self):  # for update strains in plotted figure
        if self.tsne == None:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this, plot t-SNE method')
            error_dialog.exec()
        else:
            changes = self.getChanges()
            self.tsne.onOffStrains(changes)

        ####Clustering#######

    def clusteringFunction(self):

        indices = self.tableCluster.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select  one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select  only one Experiment')
            error_dialog.exec()
        elif (self.rbNormal.isChecked() == False and self.rbSSA.isChecked() == False):
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select method type')
            error_dialog.exec()
        else:  # valid parameters
            try:
                for index in sorted(indices):  # selected experiment
                    experimentIndex = (index.row())
                    df = self.loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(self.path_pkl + experimentName)
                if self.rbNormal.isChecked() == True:  # clustering type
                    auxType = 'Normal'
                else:
                    auxType = 'SSA'
                thread = clusterProcess(dfAux, strain, repetition, fvp, lvp, auxType)  # new QRunnable clusterProcess
                thread.signals.error.connect(self.errorClustering)
                thread.signals.finished.connect(self.okClustering)
                self.loading_cluster.setMovie(self.movie)  # loading bar
                self.movie.start()
                self.threadpool.start(thread)  # start thread
            except:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error with clustering')
                error_dialog.exec()

    def errorClustering(self):
        self.movie.stop()
        self.loading_cluster.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated clustering')
        error_dialog.exec()
        logging.info('An error has occurred when calculated clustering')

    def okClustering(self, parameters):  # clustering execute successfully
        self.movie.stop()
        self.loading_cluster.clear()
        self.clusterSlider.valueChanged.disconnect()
        self.clusterSlider.setValue(0)
        self.number_cluster.setText(str(2))
        self.clusterSlider.valueChanged.connect(self.updateCluster)
        self.typeCluster.valueChanged.disconnect()
        self.typeCluster.setValue(0)
        self.typeCluster.valueChanged.connect(self.updateTypeCluster)
        strain = parameters[0]
        repetition = parameters[1]
        fvp = parameters[2]
        lvp = parameters[3]
        directory = parameters[4]
        outliers = parameters[5]
        self.clusterDF = parameters[6]  # dataframe for export function
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
        log = 'Clustering for experiment: ' + nameLog + ' was plotted'
        logging.info(log)

        self.addFunctions('Clustering', '')
        self.cluster = directory  # directory where plotted images are
        name = directory + '/Curves2.png'  # image to show
        pixmap = QPixmap(name)
        pixmap = pixmap.scaled(self.clustering1.size(), QtCore.Qt.KeepAspectRatio)
        self.clustering1.setPixmap(pixmap)  # show image
        pos = 'Positions of interest: '
        for p in outliers:
            pos = pos + str(p) + ', '
        pos = pos[:-2]
        self.addEntrydb(strain, repetition, fvp, lvp, 'Clustering', '', directory, pos)

        self.loading_cluster.setText(pos)  # set positions of interest
        self.loading_cluster.setStyleSheet('font: 75 11pt "Ubuntu Condensed";')  # style
        self.clusterSlider.valueChanged.disconnect()  # disconnect function to move slider.
        self.clusterSlider.setValue(0)
        self.number_cluster.setText(str(2))
        self.clusterSlider.valueChanged.connect(self.updateCluster)
        self.clusterSlider.setMaximum(2)

    def updateCluster(self):  # update nro cluster
        nroCluster = self.numberCluster[self.clusterSlider.value()]
        self.number_cluster.setText(str(nroCluster))

        if self.cluster == '':  # clustering not executed
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this execute cluster method')
            error_dialog.exec()
            self.clusterSlider.valueChanged.disconnect()
            self.clusterSlider.setValue(0)
            self.number_cluster.setText(str(2))
            self.clusterSlider.valueChanged.connect(self.updateCluster)
            self.clusterSlider.setMaximum(2)
        else:
            directory = self.cluster
            type = self.typeCluster.value()  # type in slider (confusion matrix or curves)
            if type == 0:
                name = directory + '/Curves' + str(nroCluster) + '.png'
            else:
                if nroCluster == 2:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Not valid Nro Clusters for Confusion Matrix')
                    error_dialog.exec()
                    self.clusterSlider.valueChanged.disconnect()
                    self.clusterSlider.setValue(1)
                    self.number_cluster.setText(str(4))
                    self.clusterSlider.valueChanged.connect(self.updateCluster)
                    name = directory + '/confMatrix4' + '.png'
                else:
                    name = directory + '/confMatrix' + str(nroCluster) + '.png'
            pixmap = QPixmap(name)
            self.clustering1.setPixmap(pixmap)  # show image

    def updateTypeCluster(self):  # to handle type(curves or Conf matrix) slider
        if self.cluster == '':  # clustering not executed
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this execute cluster method')
            error_dialog.exec()
            self.typeCluster.valueChanged.disconnect()
            self.typeCluster.setValue(0)
            self.typeCluster.valueChanged.connect(self.updateTypeCluster)
        else:
            directory = self.cluster
            nroCluster = self.numberCluster[self.clusterSlider.value()]
            if nroCluster == 2:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Confusion Matrix is Available for 4 or 6 clusters')
                error_dialog.exec()
                self.typeCluster.valueChanged.disconnect()
                self.typeCluster.setValue(0)
                self.typeCluster.valueChanged.connect(self.updateTypeCluster)
            type = self.typeCluster.value()
            if type == 0:
                name = directory + '/Curves' + str(nroCluster) + '.png'
            else:
                name = directory + '/confMatrix' + str(nroCluster) + '.png'
            pixmap = QPixmap(name)
            self.clustering1.setPixmap(pixmap)

    def exportClusterDF(self):
        if self.cluster == '':  # clustering not executed
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Plot cluster method before export result')
            error_dialog.exec()
        else:
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            fileName, _ = QFileDialog.getSaveFileName(self, directory=os.getcwd(), caption='Save result',
                                                      filter='*.csv',
                                                      options=options)
            if fileName != '':  # if user selected filename
                if '.csv' not in fileName:
                    fileName = fileName.replace('.', '') + '.csv'
                self.clusterDF.to_csv(fileName)  # export clusterDF

        ##################### PCA #################

    def pcaFunction(self):
        dfsAux = []
        dfs = []
        nameExps = []
        indices = self.tablePCA.selectionModel().selectedRows()

        if len(indices) < 2:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least two Experiments')
            error_dialog.exec()

        else:  # valid positions
            try:
                # same as t-SNE get parameters
                fvpMayor = 0
                lvpMenor = 0
                for index in sorted(indices):
                    experimentIndex = (index.row())
                    df = self.loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    if fvpMayor == 0 and lvpMenor == 0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(self.path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExps.append(experimentNametxt)
                for dataFrame in dfsAux:
                    df = keepValidPositions(dataFrame, fvpMayor, lvpMenor)
                    dfs.append(df)

                thread = pcaProcess(dfs, nameExps)  # create QRunnable pcaProcess
                thread.signals.error.connect(self.errorPCA)
                thread.signals.finished.connect(self.pcaOK)
                self.loadingPCA.setMovie(self.movie)
                self.movie.start()
                self.threadpool.start(thread)  # start thread
            except:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: An error ocurred while PCA executed')
                error_dialog.exec()

    def pcaOK(self, parameters):
        self.movie.stop()
        self.loadingPCA.clear()
        dfPuntosPCA = parameters[0]
        pcaVector = parameters[1]
        infoDF = parameters[2]
        nameExps = parameters[3]
        dfs = parameters[4]
        if self.pca == None:  # first pca executed
            boxAux = QtWidgets.QVBoxLayout(self.widget_image)  # add to layout
            self.pca = FigurePCA(dfPuntosPCA, pcaVector, infoDF, nameExps, dfs, self.widget_image, width=6, height=5,
                                 dpi=100)  # create pcaFigure to handle events and plot
            boxAux.addWidget(self.pca)
        else:
            self.pca.updatePCA(dfPuntosPCA, pcaVector, infoDF, nameExps, dfs)  # update figure
        for ne in nameExps:
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            otherexperiments = getExperiments(ne, nameExps)
            log = 'PCA executed with this experiments ' + otherexperiments
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 'PCA', otherexperiments, '')
            self.addFunctions('PCA', otherexperiments)

    def errorPCA(self):
        self.movie.stop()
        self.loadingPCA.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated PCA')
        error_dialog.exec()
        logging.info('An error has occurred when calculated PCA')

        ############ Codon Decay ##########################33

    def codonDecayFunction(self):
        indices = self.tableViewCodon.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:  # valid parameters
            try:
                th = self.threshold.toPlainText()
                fth = float(th)  # has to be float
                if fth < 0 or fth >= 1:  # valid threshold
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Please select threshold between 0 and 1')
                    error_dialog.exec()
                else:
                    for index in sorted(indices):
                        experimentIndex = (index.row())
                    df = self.loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp = str(df[2])
                    lvp = str(df[3])
                    thread = codonDecay(strain, repetition, fth, fvp, lvp)  # create QRunnable codonDecay
                    thread.signals.error.connect(self.errorPlotCD)
                    thread.signals.finished.connect(self.codonDecayOK)
                    self.loadingCD.setMovie(self.movie)
                    self.movie.start()
                    self.threadpool.start(thread)
            except Exception as e:
                if e.__class__ == FileNotFoundError:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Please enter the experiment parquet with frequencies')
                    error_dialog.exec()
                else:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Please enter an float threshold')
                    error_dialog.exec()

    def errorPlotCD(self):
        self.movie.stop()
        self.loadingCD.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated Codon Decay')
        error_dialog.exec()
        logging.info('An error has occurred when calculated Codon Decay')

    def codonDecayOK(self, parameters):
        self.movie.stop()
        self.loadingCD.clear()
        dfResult = parameters[0]
        df = parameters[1]
        th = parameters[2]
        strain = parameters[3]
        repetition = parameters[4]
        fvp = parameters[5]
        lvp = parameters[6]
        if self.condonDecay == None:  # first excution
            boxAux = QtWidgets.QVBoxLayout(self.codonDecayFrame)  # add to layout
            self.condonDecay = FigureCD(dfResult, df, th, self.codonDecayFrame, width=6, height=5,
                                        dpi=100)  # create figure for plotting and handle events
            boxAux.addWidget(self.condonDecay)
        else:
            self.condonDecay.updateCD(dfResult, df, th)  # update figure
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
        log = 'Codon Decay was plotted with threshold ' + str(th) + 'with experiment ' + nameLog
        logging.info(log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Codon Decay', str(th), '')
        self.addFunctions('Codon Decay', th)

        #### PLOT FUNCTIONS ##################

    def plotSurfaceFunction(self):  # function to plot entropy surface 3D
        indices = self.tableViewEntropySurface.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:  # valid parameters
            for index in sorted(indices):
                experimentIndex = (index.row())
            df = self.loaded_dfs[experimentIndex]
            strain = str(df[0])
            repetition = str(df[1])
            fvp = str(df[2])
            lvp = str(df[3])
            thread = plotSurface(strain, repetition, fvp, lvp)  # create QRunnable plotSurface class
            thread.signals.error.connect(self.errorPlot)
            thread.signals.finished.connect(self.plotSurfaceOK)
            self.loadingPS.setMovie(self.movie)
            self.movie.start()
            self.threadpool.start(thread)  # start thread

    def plotSurfaceOK(self, parameters):
        self.movie.stop()
        self.loadingPS.clear()
        strain = parameters[0]
        repetition = parameters[1]
        fvp = parameters[2]
        lvp = parameters[3]
        name = self.path_grafics + '/entropySurface_' + strain + '_' + repetition + '.png'
        pixmap = QPixmap(name)
        self.imageSurface.setPixmap(pixmap)  # show image
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
        log = 'Entropy surface is plotted and saved' + ' with experiment ' + nameLog
        logging.info(log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Plot Surface Entropy', '', name)
        self.addFunctions('Plot Surface Entropy', '')

    def plotPostionFunction(self):  # function to plot entropy position
        indices = self.tableView_PlotPostion.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:  # valid parameters
            try:
                position = self.textEditPosition.toPlainText()
                position = int(position)  # cast to int user position entry
                for index in sorted(indices):
                    experimentIndex = (index.row())
                df = self.loaded_dfs[experimentIndex]
                strain = str(df[0])
                repetition = str(df[1])
                fvp = str(df[2])
                lvp = str(df[3])
                thread = plotPositionEntropy(strain, repetition, position, fvp,
                                             lvp)  # QRunnable to process plotPositionEntropy
                thread.signals.error.connect(self.errorPlot)
                thread.signals.finished.connect(self.plotEntropyPositionOK)
                self.loadingPPP.setMovie(self.movie)
                self.movie.start()
                self.threadpool.start(thread)
            except:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please enter a valid position')
                error_dialog.exec()

    def plotEntropyPositionOK(self, parameters):  # succesfully process plot position
        self.movie.stop()
        self.loadingPPP.clear()
        strain = parameters[0]
        repetition = parameters[1]
        position = parameters[2]
        fvp = parameters[3]
        lvp = parameters[4]
        name = self.path_grafics + '/Entropy_' + str(position) + '_' + strain + '_' + repetition + '.png'
        pixmap = QPixmap(name)
        self.plotPositionImage.setPixmap(pixmap)  # show image
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
        log = 'Entropy is plotted and saved for position ' + str(position) + ' with experiment ' + nameLog
        logger(False, nameLog, log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Entropy times Position', position, name)
        self.addFunctions('Entropy times Position', position)

        ######## GUMBEL #################

    def gumbelFunction(self):
        indices = self.tableViewGumbel.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:  # valid parameters
            for index in sorted(indices):
                experimentIndex = (index.row())
            df = self.loaded_dfs[experimentIndex]
            strain = str(df[0])
            repetition = str(df[1])
            fvp = str(df[2])
            lvp = str(df[3])
            thread = plotGumbel(strain, repetition, fvp, lvp)  # QRunnable for gumbel process
            thread.signals.error.connect(self.errorPlot)
            thread.signals.finished.connect(self.plotGumbelOK)
            self.loadingGP.setMovie(self.movie)
            self.movie.start()
            self.threadpool.start(thread)  # start thread

    def plotGumbelOK(self, parameters):  # sucessfully gumbel process
        self.movie.stop()
        self.loadingGP.clear()
        strain = parameters[0]
        repetition = parameters[1]
        fvp = parameters[2]
        lvp = parameters[3]
        outliers = parameters[4]
        self.loadingGP.setText(outliers)  # set label por positions of interest
        self.loadingGP.setStyleSheet('font: 75 13pt "Ubuntu Condensed";')
        name = self.path_grafics + '/Gumbel' + '_' + strain + '_' + repetition + '.png'
        pixmap = QPixmap(name)  # show image
        self.imageGumbel.setPixmap(pixmap)
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')'
        log = 'Gumbel is plotted and saved' + ' with experiment ' + nameLog
        logging.info(log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Gumbel', '', name, outliers)
        self.addFunctions('Gumbel', '')

    def errorPlot(self, e):
        self.movie.stop()
        self.loadingGP.clear()
        self.loadingPPP.clear()
        self.loadingPS.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('An error has occurred when experiment was plotted')
        error_dialog.exec()
        logging.info('An error has occurred when experiment was plotted')

        ########### LV-RV Function ########################

    def lvrvFunction(self):
        dfsAux = []
        dfs = []
        nameExp = []
        indices = self.tableLVRV.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()


        elif (self.mvAverage.isChecked() == False and self.poly.isChecked() == False):
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select method type')
            error_dialog.exec()

        else:  # valid parameters
            try:
                input = self.input_lvrv.toPlainText()  # get user value for Window or Grade
                intInput = int(input)
                fvpMayor = 0
                lvpMenor = 0
                # same as t-SNE get parameters
                for index in sorted(indices):
                    experimentIndex = (index.row())
                    df = self.loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    if fvpMayor == 0 and lvpMenor == 0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(self.path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExp.append(experimentNametxt)
                for dataFrame in dfsAux:
                    df = keepValidPositions(dataFrame, fvpMayor, lvpMenor)
                    dfs.append(df)
                if self.mvAverage.isChecked() == True:  # type LVRV
                    thread = lvrv_average(dfs, intInput, nameExp)  # Qrunnable to process lvrv moving average
                    thread.signals.finished.connect(self.lvrvOK)
                    thread.signals.error.connect(self.lvrvError)

                    self.loadingLVRV.setMovie(self.movie)
                    self.movie.start()
                    self.threadpool.start(thread)  # start thread
                else:
                    thread = lvrv_poly(dfs, intInput, nameExp)  # Qrunnable to process lvrv poly
                    thread.signals.finished.connect(self.lvrvOK)
                    thread.signals.error.connect(self.lvrvError)
                    self.loadingLVRV.setMovie(self.movie)
                    self.movie.start()
                    self.threadpool.start(thread)  # start thread
            except:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please enter an integer')
                error_dialog.exec()

    def lvrvError(self):  # error in lvrv process
        self.movie.stop()
        self.loadingLVRV.clear()
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated LV-RV')
        error_dialog.exec()
        logging.info('An error has occurred when calculated LV-RV')

    def lvrvOK(self, parameters):  # sucess in lvrv process
        self.movie.stop()
        self.loadingLVRV.clear()
        dfs = parameters[0]
        typeLVRV = parameters[1]
        nameExps = parameters[2]
        param = parameters[3]
        if self.lvrv == None:  # first execution
            boxAux = QtWidgets.QVBoxLayout(self.lvrv_frame)  # add to layout
            self.lvrv = FigureLVRV(dfs, self.lvrv_frame, nameExps, typeLVRV, param, width=6, height=5,
                                   dpi=100)  # create lvrv figure to plot and handle events
            boxAux.addWidget(self.lvrv)
        else:
            self.lvrv.updatelvrv(dfs, nameExps, typeLVRV, param)  # update figure
        for ne in nameExps:
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            otherexperiments = getExperiments(ne, nameExps)
            log = typeLVRV + ' LV-RV was plotted with parameter ' + str(
                param) + ' with this experiments: ' + otherexperiments
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, typeLVRV + ' LV-RV',
                            otherexperiments + ' with parameter = ' + str(param), '')
            self.addFunctions(typeLVRV + ' LV-RV', otherexperiments + ' with parameter = ' + str(param))

        # functions for move stackedWidget(main buttons)

    def entropyPosition(self):
        self.stackedWidget.setCurrentIndex(1)  # plotEntropyPosition

    def entropySurface(self):
        self.stackedWidget.setCurrentIndex(2)

    def codonDecayWidget(self):
        self.stackedWidget.setCurrentIndex(4)

    def tsne(self):
        self.stackedWidget.setCurrentIndex(5)

    def pcaWidget(self):
        self.stackedWidget.setCurrentIndex(3)

    def gumbelWidget(self):
        self.stackedWidget.setCurrentIndex(6)

    def LVRV(self):
        self.stackedWidget.setCurrentIndex(7)

    def Clustering(self):
        self.stackedWidget.setCurrentIndex(8)

    def returnHome(self):
        self.stackedWidget.setCurrentIndex(0)

        # functions to export

    def exportTSNE(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName, _ = QFileDialog.getSaveFileName(self, directory=os.getcwd(), caption='Save experiment',
                                                  filter='*.csv',
                                                  options=options)
        if fileName != '':
            if '.csv' not in fileName:  # csv not in filename
                fileName = fileName.replace('.', '') + '.csv'
            if self.tsne == None:  # t-SNE not executed
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please Plot t-SNE before export')
                error_dialog.exec()
            else:
                nameExp, normal, lr, perp, tsneType = self.tsne.exportCSV(
                    fileName)  # info from export. This is necessary for Data Base
                for ne in nameExp:
                    auxStrain = ne.split("_")
                    strain = auxStrain[0]
                    auxRep = auxStrain[1].split("(")
                    repetition = auxRep[0]
                    auxfvp = auxRep[1].split("-")
                    fvp = auxfvp[0]
                    lvp = auxfvp[1].split(")")[0]
                    otherexperiments = getExperiments(ne, nameExp)
                    if normal:
                        log = tsneType + ' t-SNE executed with this experiments: ' + otherexperiments

                        logging.info(log)
                        self.addEntrydb(strain, repetition, fvp, lvp, 't-SNE ' + tsneType, otherexperiments, fileName)
                    else:
                        log = tsneType + ' t-SNE executed with perplexity = ' + str(
                            perp) + ' and learning rate = ' + str(lr) + ' with this experiments: ' + otherexperiments
                        logging.info(log)
                        self.addEntrydb(strain, repetition, fvp, lvp,
                                        't-SNE ' + tsneType + ' with perp=' + str(perp) + ' and lr=' + str(lr),
                                        otherexperiments, fileName)

    def importTSNE(self):
        try:
            nameExps = []
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            (fileName, aux) = QFileDialog.getOpenFileName(self, directory=os.getcwd(), caption='Select a .csv File',
                                                          filter='*.csv', options=options)
            if fileName != "":  # select experiment
                df = pd.read_csv(fileName)
                strains = df.Strain.unique()  # get strains from csv
                self.addSlidersTSNE(strains)  # add on/off sliders
                columns = df.columns.unique()  # get columns information
                if 'Position' in columns:  # get type of t-SNE
                    tsneType = 'Positions'
                else:
                    tsneType = 'Passages'
                perps = df.Perp.unique()
                lrs = df.LR.unique()

                if len(perps) == 1 and len(
                        lrs) == 1:  # if only had info of add lr and perp(not from normally execution)
                    perp = perps[0]
                    lr = lrs[0]
                else:
                    lr = None
                    perp = None

                if self.tsne == None:  # if is first execution
                    boxAux = QtWidgets.QVBoxLayout(self.tsneFrame)  # add to layout
                    self.tsne = FigureTSNE(None, df, tsneType, nameExps, self.tsneFrame, lr, perp, width=6, height=5,
                                           dpi=100)  # create figure
                    boxAux.addWidget(self.tsne)
                else:
                    self.tsne.newtSNE(None, df, tsneType, nameExps, lr, perp)  # update figure
                self.perplexitySlider.valueChanged.disconnect()
                self.lrSlider.valueChanged.disconnect()
                if tsneType == 'Passages':  # set sliders info for passages type
                    self.perplexitySlider.setValue(0)
                    self.lrSlider.setValue(0)
                    self.perplexitySlider.setMaximum(3)
                    self.lrSlider.setMaximum(3)
                    self.perAux = [3, 6, 12, 15]
                    self.lrAux = [10, 100, 300, 500]
                else:  # set sliders info for positions type

                    self.perplexitySlider.setMaximum(1)
                    self.lrSlider.setMaximum(1)
                    self.perAux = [3, 12]
                    self.lrAux = [10, 300]

                    self.perplexitySlider.setValue(0)
                    self.lrSlider.setValue(0)

                self.perplexitySlider.valueChanged.connect(self.updateTSNE)
                self.lrSlider.valueChanged.connect(self.updateTSNE)
        except:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please import a valid t-SNE experiment CSV')
            error_dialog.exec()

    def exportLVRV(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName, _ = QFileDialog.getSaveFileName(self, directory=os.getcwd(), caption='Save experiment',
                                                  filter='*.csv',
                                                  options=options)
        if fileName != '':
            if '.csv' not in fileName:
                fileName = fileName.replace('.', '') + '.csv'
            if self.lvrv == None:  # lvrv not executed
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please getLastResultsPlot LV-RV before export')
                error_dialog.exec()
            else:
                nameExps, typeLVRV, param = self.lvrv.exportCSV(fileName)  # get info for DB
                for ne in nameExps:
                    auxStrain = ne.split("_")
                    strain = auxStrain[0]
                    auxRep = auxStrain[1].split("(")
                    repetition = auxRep[0]
                    auxfvp = auxRep[1].split("-")
                    fvp = auxfvp[0]
                    lvp = auxfvp[1].split(")")[0]

                    otherexperiments = getExperiments(ne, nameExps)

                    log = typeLVRV + ' LV-RV was plotted with parameter ' + str(
                        param) + ' with this experiments: ' + otherexperiments
                    logging.info(log)
                    self.addEntrydb(strain, repetition, fvp, lvp, typeLVRV + ' LV-RV',
                                    otherexperiments + ' with parameter = ' + str(param), fileName)

    def importLVRV(self):
        try:

            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            (fileName, aux) = QFileDialog.getOpenFileName(self, directory=os.getcwd(), caption='Select a .csv File',
                                                          filter='*.csv',
                                                          options=options)

            if fileName != "":  # user select csv
                df = pd.read_csv(fileName)
                lvrvType = df.type.unique()[0]  # get type

                if self.lvrv == None:  # first execution
                    boxAux = QtWidgets.QVBoxLayout(self.lvrv_frame)
                    self.lvrv = FigureLVRV(df, self.lvrv_frame, lvrvType, None, width=6, height=5,
                                           dpi=100)  # create figure
                    boxAux.addWidget(self.lvrv)
                else:
                    self.lvrv.updatelvrv(df)  # update figure
        except:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please import a valid LV-RV experiment CSV')
            error_dialog.exec()

    def exportPCA(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName, _ = QFileDialog.getSaveFileName(self, directory=os.getcwd(), caption='Save experiment',
                                                  filter='*.csv',
                                                  options=options)
        if fileName != '':
            if '.csv' not in fileName:
                fileName = fileName.replace('.', '') + '.csv'
            if self.pca == None:  # not pca executed
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please Plot PCA before export')
                error_dialog.exec()
            else:
                nameExps = self.pca.exportPCA(fileName)  # get info for DB
                for ne in nameExps:
                    auxStrain = ne.split("_")
                    strain = auxStrain[0]
                    auxRep = auxStrain[1].split("(")
                    repetition = auxRep[0]
                    auxfvp = auxRep[1].split("-")
                    fvp = auxfvp[0]
                    lvp = auxfvp[1].split(")")[0]
                    otherexperiments = getExperiments(ne, nameExps)
                    log = 'PCA executed with this experiments ' + otherexperiments
                    logging.info(log)
                    self.addEntrydb(strain, repetition, fvp, lvp, 'PCA', otherexperiments, fileName)

    def importPCA(self):
        try:

            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            (fileName, aux) = QFileDialog.getOpenFileName(self, directory=os.getcwd(), caption='Select a .csv File',
                                                          filter='*.csv',
                                                          options=options)

            if fileName != "":  # user select csv
                df = pd.read_csv(fileName)
                if self.pca == None:  # first execution
                    boxAux = QtWidgets.QVBoxLayout(self.widget_image)
                    self.pca = FigurePCA(None, None, None, None, None, self.widget_image, df, width=6,
                                         height=5, dpi=100)  # create figure
                    boxAux.addWidget(self.pca)
                else:
                    self.pca.updatePCA(None, None, None, None, None, df)  # update figure
        except:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please import a valid PCA experiment CSV')
            error_dialog.exec()

        # data base functions

    def getLastResults(self):
        if self.Online:  # online mode
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            fileName, _ = QFileDialog.getSaveFileName(self, directory=os.getcwd(), caption='Save Results',
                                                      filter='*.csv',
                                                      options=options)
            if fileName != '':
                if '.csv' not in fileName:
                    fileName = fileName.replace('.', '') + '.csv'

                vevExperiments = self.db['ResultsExperiments']  # get table
                results = vevExperiments.find({"_id.user": self.namePC})  # find users results
                csv_columns = ['Strain', 'Repetition', 'fvp', 'lvp', 'function', 'parameters', 'result',
                               'outliers']  # csv columns
                try:
                    csvfile = open(fileName, 'w')  # create csvfile
                    writer = csv.DictWriter(csvfile, fieldnames=csv_columns)  # set columns
                    writer.writeheader()
                    for experiments in results:  # for every user result experiments in ResultsExperiments
                        data = {"Strain": experiments["_id"]["strain"], "Repetition": experiments["_id"]["repetition"],
                                "fvp": experiments["_id"]["fvp"],
                                "lvp": experiments["_id"]["lvp"], "function": experiments["_id"]["expName"],
                                "parameters": experiments["params"],
                                "result": experiments["result"]}
                        # get data info and add to dict
                        try:
                            data["outliers"] = experiments[
                                "Interest Positions"]  # outliers is not in every entry(added later)
                        except:
                            data["outliers"] = ''
                        writer.writerow(data)  # write data
                    csvfile.close()
                except:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('An Error ocurred while getting last results')
                    error_dialog.exec()
        else:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: VEV Application is in Offline mode')
            error_dialog.exec()

    def addFunctions(self, funcName, params):  # add entry in database for functions
        if self.Online:  # online mode
            functionsCollection = self.db['functions']
            functionEntry = {"_id": funcName, "parameters": params}
            validation = self.validateEntry(functionEntry, 'Function')  # validate schema
            if validation:
                entryD = functionsCollection.find_one({"_id": funcName})
                if entryD == None:  # insert
                    functionsCollection.insert_one(functionEntry)
                else:
                    functionsCollection.update_one({'_id': funcName}, {"$set": functionEntry}, upsert=False)

    def addExperiment(self, strain, repetition, fvp, lvp):  # add entry in database for experiments
        if self.Online:  # online mode
            experimentsCollection = self.db['experiments']
            fvp = int(fvp)
            lvp = int(lvp)
            repetition = int(repetition)
            experimentEntry = {"_id": {"user": self.namePC, "strain": strain, "repetition": repetition, "fvp": fvp,
                                       "lvp": lvp}}
            validation = self.validateEntry(experimentEntry, 'Experiment')  # validate schema
            if validation:
                entryD = experimentsCollection.find_one({"_id": experimentEntry["_id"]})
                if entryD == None:  # insert
                    experimentsCollection.insert_one(experimentEntry)

    def addEntrydb(self, strain, repetition, fvp, lvp, expName, params, result, outliers=None):
        if self.Online:  # online mode
            fvp = int(fvp)
            lvp = int(lvp)
            repetition = int(repetition)
            vevExp = {"_id": {"user": self.namePC, "strain": strain, "repetition": repetition, "fvp": fvp,
                              "lvp": lvp, "expName": expName}, "params": str(params), "result": result}
            validation = self.validateEntry(vevExp, 'ResultsExperiments')  # validate schema
            if validation:
                vevExperiments = self.db['ResultsExperiments']
                vevExp['timestamp'] = datetime.now().strftime("%d-%m-%Y,%H:%M:%S")
                if outliers is not None:  # gumbel or clustering
                    vevExp['Interest Positions'] = outliers

                entryD = vevExperiments.find_one({"_id": vevExp["_id"]})
                if entryD == None:  # insert
                    vevExperiments.insert_one(vevExp)
                else:  # update
                    vevExperiments.update_one({'_id': vevExp["_id"]}, {"$set": vevExp}, upsert=False)

    def addUser(self):
        if self.Online:  # online mode
            userCollection = self.db['users']
            entryD = userCollection.find_one({"_id": self.namePC})
            user = {"_id": self.namePC}
            validation = self.validateEntry(user, 'User')  # validate schema
            if validation:
                if entryD == None:  # insert
                    userCollection.insert_one(user)

    def validateEntry(self, entry, collection):  # validation schema with json-schema
        # first need to select proper schema
        try:
            if collection == 'ResultsExperiments':
                with open('Schema experiments results.json') as j:
                    schema = json.load(j)
            elif collection == 'Experiment':
                with open('Schema experiments.json') as j:
                    schema = json.load(j)
            elif collection == 'Function':
                with open('Schema function.json') as j:
                    schema = json.load(j)
            else:
                with open('Schema user.json') as j:
                    schema = json.load(j)
            validate(instance=entry, schema=schema)  # validate entry
            return True
        except:
            return False
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Invalid entry. Not fulfill schema')
            error_dialog.exec()


def main():
    args = sys.argv
    app = QtWidgets.QApplication([])
    args = [x.upper() for x in args]  # upper args to avoid case sensitive
    if 'OFFLINE' in args:
        win = Interface('OFFLINE')
    else:
        win = Interface()
    win.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()




def plotPopularDecay(df):
    # gets values of the first passage
    minPassage = df['Passage'].min()

    passages = df['Passage'].unique()  # get passages

    info_df = str(df['info'].iloc[0])  # get info
    strain_df = info_df.split(',')[0]
    repetition_df = info_df.split(',')[1]

    posVal = df.Position.unique()  # get unique positions

    # look for the values of the probabilities for the first passage
    dfFirstPassage = df[df.Passage.eq(minPassage)]
    dfFirstPassage_val = dfFirstPassage[dfFirstPassage.Position.isin(posVal)].copy()
    dfFirstPassage_val_aux = dfFirstPassage_val.drop(
        ['Position', 'Coverage', 'Passage', 'Strain', 'Repetition', 'info'], axis=1)
    maxColumnsCodon = dfFirstPassage_val_aux.idxmax(axis=1)
    dfFirstPassage_val['MaxFreqCodon'] = maxColumnsCodon
    dfFirstPassage_val = dfFirstPassage_val[['Position', 'MaxFreqCodon']]
    return dfFirstPassage_val


    
    
    



from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
from src.Clustering import getClustering, plotClustersCurves, plotClustersDimRed, plot_confusion_matrix,getClusteringSSA
from sklearn.metrics import confusion_matrix
import numpy as np
from pathlib import Path
import os
import warnings
class clusterSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal()


class clusterProcess(QRunnable):

    def __init__(self,dataframe,strain,repetition,fvp,lvp,typeCluster): #create Class
        super(clusterProcess, self).__init__()
        self.df = dataframe
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.type = typeCluster
        self.signals = clusterSignals()


    @pyqtSlot()
    def run(self):

        try:
            pathPip = str(os.path.dirname(np.__file__)).split('numpy')[0]
            metrica="dtw" #metric use in clustering
            if self.type == 'Normal': #get type
                data_array, d_T, list_pos, list_bary = getClustering([self.df], metrica, perp=3, lr=100) #call getClustering
            else:
                data_array, d_T, list_pos, list_bary = getClusteringSSA([self.df], metrica, perp=3, lr=100) #call clusteringSSA
            threshold = int(len(d_T.Position.unique()) * 0.01) #threshold for number of elements in clusters
            dfAux = d_T[['6','Position']] #get information of positions clusters labels with 6 clusters
            dfAux = dfAux.rename(columns={"6":"Label","Position":"Position"}) #change column for 6 cluster labels to labels
            labels = dfAux.Label.unique() #get labels
            outliers = []
            for l in labels: #for every label
                df = dfAux[dfAux.Label.eq(l)] #get df with label l
                listOut = df['Position'].values.tolist()
                if len(listOut) <= threshold: #if cluster has less or equal elements than threshold, save this positions
                    outliers.append(listOut)
            outliers = [item for sublist in outliers for item in sublist] #get only 1 list
            outliers = list(map(int, outliers)) #transform to int
            outliers.sort() #sort elements
            directory = pathPip+'src/plot_images/' + str(self.strain) + '_'+ str(self.repetition) + '(' +str(self.fvp) + '-' + str(self.lvp)+')'
            Path(directory).mkdir(parents=True, exist_ok=True) #if directory not exists, create
            clusters = [2,4,6]
            for i in range(3):

                plotClustersCurves(data_array, list_pos[i], list_bary[i],clusters[i],directory) #plot cluster curves for every cluster
                if i == 1:
                    target_clusters = ['0', '1', '2', '3']
                    c_2 = np.array(d_T[['2']])
                    c_4 = np.array(d_T[['4']])
                    cnf_matrix = confusion_matrix(c_4, c_2)
                    c_m = cnf_matrix[:, :-2]
                    plot_confusion_matrix(c_m,clusters[i],directory, classes=target_clusters, title='Confusion matrix 2 and 4 clusters') #plot confusion matrix

                elif i == 2:
                    target_clusters = ['0', '1', '2', '3', '4', '5']
                    c_6 = np.array(d_T[['6']])
                    cnf_matrix = confusion_matrix(c_6, c_4)
                    c_m = cnf_matrix[:, :-2]
                    plot_confusion_matrix(c_m,clusters[i],directory, classes=target_clusters, title='Confusion matrix 4 and 6 clusters')

            d_T = d_T.rename(columns={"6": "Label 6 Cluster", "4": "Label 4 Cluster","2": "Label 2 Cluster"})  # change column for 6 cluster labels to labels

        except:
            self.signals.error.emit()

        finally:
            self.signals.finished.emit([self.strain,self.repetition,self.fvp,self.lvp,directory,outliers,d_T])  # Done





from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
import pandas as pd
from src.GetEntropySurface import getEntropySurface
from src.PlotEntropyPosition import plotEntropyPosition
from src.calculateGumbel import gumbelPlot
from src.PlotPopularDecay import plotPopularDecay
import os


class plotSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal(object)

class plotSurface(QRunnable): #process in other thread plot surface

    def __init__(self,strain,repetition,fvp,lvp):
        super(plotSurface, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()
    @pyqtSlot()
    def run(self):
        try:
            pathPip = str(os.path.dirname(pd.__file__)).split('pandas')[0]
            path_pkl = pathPip + 'src/data_experiments/'
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'  # get experiment
            df = pd.read_parquet(path_pkl + experimentName)  # read experiment
            print('2')
            getEntropySurface(df, self.strain, self.repetition)  # calculate entropy surface
            print('1')
        except Exception as e:
            print(e)
            self.signals.error.emit(e)
        finally:
            self.signals.finished.emit([self.strain, self.repetition, self.fvp, self.lvp])  # done, send info to main thread

class plotPositionEntropy(QRunnable): #process in other thread plot entropy * position

    def __init__(self,strain,repetition,position,fvp,lvp):
        super(plotPositionEntropy, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.position = position
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            pathPip = str(os.path.dirname(pd.__file__)).split('pandas')[0]
            path_pkl = pathPip + 'src/data_experiments/'
            intposition = int(self.position)
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'  # get experiment
            df = pd.read_parquet(path_pkl + experimentName)  # read dataframe
            plotEntropyPosition(intposition, df, self.strain, self.repetition, 'red')  # plot entropy position
        except:
            self.signals.error.emit()
        finally:
            self.signals.finished.emit([self.strain, self.repetition, self.position, self.fvp, self.lvp])  # done, send to main thread

class plotGumbel(QRunnable): #process in other thread, calculate gumbel
    def __init__(self,strain,repetition,fvp,lvp):
        super(plotGumbel, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            pathPip = str(os.path.dirname(pd.__file__)).split('pandas')[0]
            path_pkl = pathPip + 'src/data_experiments/'
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'
            df = pd.read_parquet(path_pkl + experimentName)  # get dataframe
            outliers = gumbelPlot(df, self.strain, self.repetition)  # plot gumbel and get outliers
        except Exception as e:
            self.signals.error.emit(e)
        finally:
            self.signals.finished.emit(
                [self.strain, self.repetition, self.fvp, self.lvp, outliers])  # done, send to main thread


class codonDecay(QRunnable): #process in other thread. Plot codon decay
    def __init__(self,strain,repetition,th,fvp,lvp):
        super(codonDecay, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.th = th
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            pathPip = str(os.path.dirname(pd.__file__)).split('pandas')[0]
            path_pkl = pathPip + 'src/data_experiments/'
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ').parquet'
            df = pd.read_parquet(path_pkl + experimentName)  # get dataframe experiment
            dfReturn = plotPopularDecay(df, )  # calculate codondecay info, get dataframe with this info
        except:
            self.signals.error.emit("ERROR")
        finally:
            self.signals.finished.emit(
                [dfReturn, df, self.th, self.strain, self.repetition, self.fvp, self.lvp])  # done, send to main thread


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.ndimage.filters import uniform_filter1d
from src.Format_DF import totalExperimentsToPassages, formatStrains, experimentToPositions



def movingAverage_1D(arr, n=3):
    y_f = uniform_filter1d(arr, mode='constant', size=n)
    r=int(np.floor(n/2)+1)
    for i in range(r-1):
        d=((np.floor(n/2)+1)+i)
        y_f[i]=n*y_f[i]/d


    for i in range(y_f.shape[0]-r+1,y_f.shape[0]):
        d=y_f.shape[0]-i+np.floor(n/2)
        y_f[i]=n*y_f[i]/d

    return y_f


def movingAverage(arr, n=3):

    arr_MA = np.zeros_like(arr)

    for i in range(arr.shape[0]):
        arr_MA[i]=movingAverage_1D(arr[i],n)
    return arr_MA


def decomposition_PM_LV_RV(df, window):

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    arr_Low = movingAverage(ar_Data, n=window)
    arr_High=np.abs(ar_Data-arr_Low)

    ar_Data_Sum=np.sum(ar_Data, axis=1)
    arr_Low_Sum=np.sum(arr_Low, axis=1)
    arr_HighSum=np.sum(arr_High, axis=1)

    data_Aux=pd.DataFrame(columns=["Info", "Position", "Strain", "Repetition", "Entropy_Acum", "Entropy_Low", "Entropy_High"])

    data_Aux['Info']=ar_Info
    data_Aux['Position']=df_I.Position
    data_Aux['Strain']=df_I.Strain
    data_Aux['Repetition']=df_I.Repetition
    data_Aux['Entropy_Acum']=ar_Data_Sum
    data_Aux['Entropy_Low']=arr_Low_Sum
    data_Aux['Entropy_High']=arr_HighSum

    return data_Aux



def decomposition_Poly_LV_RV(df, grade): #lv_rv poly

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df) #get info

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"] #split info column

    passages=df['Passage'].unique() #get passages

    passages = list(map(int, passages)) #cast to int

    arr_Low = np.zeros_like(ar_Data)

    for i in range(ar_Data.shape[0]):
        z = np.polyfit(passages, ar_Data[i], grade) #Least squares
        p = np.poly1d(z)
        arr_Low[i]=p(passages)

    arr_High=np.abs(ar_Data-arr_Low)

    ar_Data_Sum=np.sum(ar_Data, axis=1)
    arr_Low_Sum=np.sum(arr_Low, axis=1)
    arr_HighSum=np.sum(arr_High, axis=1)

    data_Aux=pd.DataFrame(columns=["Info", "Position", "Strain", "Repetition", "Entropy_Acum", "Entropy_Low", "Entropy_High"])

    data_Aux['Info']=ar_Info
    data_Aux['Position']=df_I.Position
    data_Aux['Strain']=df_I.Strain
    data_Aux['Repetition']=df_I.Repetition
    data_Aux['Entropy_Acum']=ar_Data_Sum
    data_Aux['Entropy_Low']=arr_Low_Sum
    data_Aux['Entropy_High']=arr_HighSum

    return data_Aux
